@@ -120,6 +120,9 @@ class Producer(object):
         if compression is None:
             compression = self.compression
 
+        if isinstance(exchange, Exchange):
+            exchange = exchange.name
+
         body, content_type, content_encoding = self._prepare(
                 body, serializer, content_type, content_encoding,
                 compression, headers)
@@ -142,10 +145,11 @@ class Producer(object):
         return self
 
     def __exit__(self, *exc_info):
-        self.close()
+        self.release()
 
-    def close(self):
+    def release(self):
         pass
+    close = release
 
     def _prepare(self, body, serializer=None,
             content_type=None, content_encoding=None, compression=None,
@@ -281,6 +285,8 @@ class Consumer(object):
         """Register consumer on server.
 
         """
+        if not self.queues:
+            return
         if no_ack is None:
             no_ack = self.no_ack
         H, T = self.queues[:-1], self.queues[-1]
